/**
 * Copyright Copyright 2007-8 Simon Andrews
 *
 *    This file is part of StackMeasure.
 *
 *    StackMeasure is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    StackMeasure is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with StackMeasure; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import ij.IJ;
import ij.io.FileSaver;

import javax.swing.ButtonGroup;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JSplitPane;
import javax.swing.filechooser.FileFilter;


public class CombinedResultsWindow extends JFrame {

	private JSplitPane leftRightPane;
	private JSplitPane topBottomPane;
	private final GroupCollection collection;
	
	private int groupBy = DistanceMeasureSet.GROUP_BY_COLOUR;
	private int measureFrom = DistanceMeasureSet.MEASURE_FROM_CENTRE;
	
	private SpatialResultsViewer srv = null;
	private VolumeDisplay vd = null;
	private DistanceDisplay gdd = null;
	private DistanceMeasureSet dms = null;
	
	public CombinedResultsWindow (GroupCollection collection) {
				
		this.collection = collection;

		setTitle("Stack Measure");
		
		// There is a problem with Java3D where just doing a dispose on
		// close results in the objects staying active.  The program
		// can't then exit and the memory never releases.  We therefore
		// have to exit on close.
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				
		setJMenuBar(new ResultsMenu());
		
		setSize(800,600);
		setLocationRelativeTo(IJ.getInstance());
		
		leftRightPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		topBottomPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		leftRightPane.setRightComponent(topBottomPane);
		
		srv = new SpatialResultsViewer(collection);
		leftRightPane.setLeftComponent(srv);
		
		vd = new VolumeDisplay(collection);
		srv.addListener(vd);
		
		topBottomPane.setBottomComponent(vd);
		dms = new DistanceMeasureSet(collection,groupBy,measureFrom);
		gdd = new DistanceDisplay(dms);
		topBottomPane.setTopComponent(gdd);
		gdd.addDistanceSelectionListener(vd);
		
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(leftRightPane,BorderLayout.CENTER);
				
		setVisible(true);

		leftRightPane.setDividerLocation(0.5);
		topBottomPane.setDividerLocation(0.5);
		
		collection.getFiles().getMergedImage().show();
		
	}
	
	private class ResultsMenu extends JMenuBar implements ActionListener {
		
		public ResultsMenu () {
			JMenu fileMenu = new JMenu("File");
			JMenu fileSave = new JMenu("Save");
			
			JMenuItem fileSaveSpatialHTML = new JMenuItem("Spatial Group Results HTML");
			fileSaveSpatialHTML.setActionCommand("save_spatial_html");
			fileSaveSpatialHTML.addActionListener(this);
			fileSave.add(fileSaveSpatialHTML);

			JMenuItem fileSaveSpatialText = new JMenuItem("Spatial Group Results Text");
			fileSaveSpatialText.setActionCommand("save_spatial_text");
			fileSaveSpatialText.addActionListener(this);
			fileSave.add(fileSaveSpatialText);

			JMenuItem fileSaveDistanceHTML = new JMenuItem("Distance Measures HTML");
			fileSaveDistanceHTML.setActionCommand("save_distance_html");
			fileSaveDistanceHTML.addActionListener(this);
			fileSave.add(fileSaveDistanceHTML);
			
			JMenuItem fileSaveDistanceText = new JMenuItem("Distance Measures Text");
			fileSaveDistanceText.setActionCommand("save_distance_text");
			fileSaveDistanceText.addActionListener(this);
			fileSave.add(fileSaveDistanceText);
			
			JMenuItem fileSaveCompositeImage = new JMenuItem("Composite Image");
			fileSaveCompositeImage.setActionCommand("save_image");
			fileSaveCompositeImage.addActionListener(this);
			fileSave.add(fileSaveCompositeImage);
			
			fileMenu.add(fileSave);
			fileMenu.addSeparator();
			
			JMenuItem fileClose = new JMenuItem("Close");
			fileClose.setActionCommand("close");
			fileClose.addActionListener(this);
			fileMenu.add(fileClose);
			
			add(fileMenu);
			
			JMenu measuresMenu = new JMenu("Measures");

			JMenu groupColours = new JMenu("Group Colours");
			JRadioButtonMenuItem groupColoursGroup = new JRadioButtonMenuItem("By Colour");
			groupColoursGroup.setActionCommand("group_colour");
			groupColoursGroup.addActionListener(this);
			groupColoursGroup.setSelected(true);
			groupColours.add(groupColoursGroup);
			JRadioButtonMenuItem groupColoursSeparate = new JRadioButtonMenuItem("Keep Separate");
			groupColoursSeparate.setActionCommand("group_separate");
			groupColoursSeparate.addActionListener(this);
			groupColours.add(groupColoursSeparate);
			
			ButtonGroup b = new ButtonGroup();
			b.add(groupColoursGroup);
			b.add(groupColoursSeparate);
			
			measuresMenu.add(groupColours);
			
			JMenu measuresMeasureFrom = new JMenu("Measure from");
			
			JRadioButtonMenuItem measurePixel = new JRadioButtonMenuItem("Closest pixel");
			measurePixel.setActionCommand("measure_pixel");
			measurePixel.setSelected(true);
			measurePixel.addActionListener(this);
			measuresMeasureFrom.add(measurePixel);
			
			JRadioButtonMenuItem measureCentre = new JRadioButtonMenuItem("Centre of mass");
			measureCentre.setActionCommand("measure_centre");
			measureCentre.addActionListener(this);
			measuresMeasureFrom.add(measureCentre);
			
			ButtonGroup b2 = new ButtonGroup();
			b2.add(measurePixel);
			b2.add(measureCentre);
			
			measuresMenu.add(measuresMeasureFrom);
			
			add(measuresMenu);
			
			JMenu viewMenu = new JMenu("View");
			JMenuItem viewComposite = new JMenuItem("View Composite Image");
			viewComposite.setActionCommand("composite");
			viewComposite.addActionListener(this);
			viewMenu.add(viewComposite);
			
			add(viewMenu);
		}

		public void actionPerformed(ActionEvent ae) {
			String command = ae.getActionCommand();
			
			if (command.equals("close")) {
				setVisible(false);
				dispose();
			}
			else if (command.equals("group_separate")) {
				groupBy = DistanceMeasureSet.GROUP_BY_SPATIAL;
				dms.ChangeMeasureOptions(groupBy, measureFrom);
			}
			else if (command.equals("group_colour")) {
				groupBy = DistanceMeasureSet.GROUP_BY_COLOUR;
				dms.ChangeMeasureOptions(groupBy, measureFrom);
			}
			else if (command.equals("measure_pixel")) {
				measureFrom = DistanceMeasureSet.MEASURE_BY_PIXEL;
				dms.ChangeMeasureOptions(groupBy, measureFrom);
			}
			else if (command.equals("measure_centre")) {
				measureFrom = DistanceMeasureSet.MEASURE_FROM_CENTRE;
				dms.ChangeMeasureOptions(groupBy, measureFrom);
			}
			else if (command.equals("composite")) {
				collection.getFiles().getMergedImage().hide();
				collection.getFiles().getMergedImage().show();
			}
			else if (command.equals("save_spatial_html")) {
				File f = getSaveFile(new String [] {"html","htm"},"HTML files");
				if (f != null) {
					try {
						collection.saveSpatialHTMLTable(f);
					} 
					catch (IOException e) {
						JOptionPane.showMessageDialog(this, "Failed to save table: "+e.getMessage(), "Save error", JOptionPane.ERROR_MESSAGE);
						e.printStackTrace();
					}					
				}
			}
			else if (command.equals("save_spatial_text")) {
				File f = getSaveFile(new String [] {"txt"}, "Text files");
				if (f != null) {
					try {
						collection.saveSpatialTextTable(f);
					} 
					catch (IOException e) {
						JOptionPane.showMessageDialog(this, "Failed to save table: "+e.getMessage(), "Save error", JOptionPane.ERROR_MESSAGE);
						e.printStackTrace();
					}					
				}
			}
			else if (command.equals("save_distance_html")) {
				File f = getSaveFile(new String [] {"html","htm"}, "HTML files");
				if (f != null) {
					try {
						dms.saveHTMLTable(f);
					}
					catch (IOException e) {
						JOptionPane.showMessageDialog(this, "Failed to save table: "+e.getMessage(), "Save error", JOptionPane.ERROR_MESSAGE);
						e.printStackTrace();
					}					
				}
			}
			else if (command.equals("save_distance_text")) {
				File f = getSaveFile(new String [] {"txt"}, "Text files");
				if (f != null) {
					try {
						dms.saveTextTable(f);
					}
					catch (IOException e) {
						JOptionPane.showMessageDialog(this, "Failed to save table: "+e.getMessage(), "Save error", JOptionPane.ERROR_MESSAGE);
						e.printStackTrace();
					}					
				}
			}
			else if (command.equals("save_image")) {
				File f = getSaveFile(new String [] {"tiff","tif"}, "TIFF files");
				if (f != null) {
					try {
						//TODO: Figure out why the first frame is blank in the saved TIFF (grrr!).
						FileSaver fs = new FileSaver(collection.getFiles().getMergedImage());
						fs.saveAsTiffStack(f.getAbsolutePath());
					}
					catch (Exception e) {
						JOptionPane.showMessageDialog(this, "Failed to save table: "+e.getMessage(), "Save error", JOptionPane.ERROR_MESSAGE);
						e.printStackTrace();
					}					
				}
			}
			else {
				System.err.println("Unkown action '"+command+"'");
			}
			
		}
		
		private File getSaveFile (final String [] extensions, final String description) {
			JFileChooser chooser = new JFileChooser();
			chooser.setMultiSelectionEnabled(false);
			chooser.setFileFilter(new FileFilter() {

				public String getDescription() {
					return description;
				}

				public boolean accept(File f) {
					if (f.isDirectory()) {
						return true;
					}
					for (int i=0;i<extensions.length;i++) {
						if (f.getName().toLowerCase().endsWith(extensions[i].toLowerCase())) {
							return true;
						}
					}
					return false;
				}

			});

			int result = chooser.showSaveDialog(this);
			if (result == JFileChooser.CANCEL_OPTION) return null;

			File file = chooser.getSelectedFile();
			boolean extOK = false;
			for (int i=0;i<extensions.length;i++) {
				if (file.getName().toLowerCase().endsWith(extensions[i].toLowerCase())) {
					extOK = true;
					break;
				}
			}
			if (! extOK) {
				file = new File(file.getPath()+"."+extensions[0]);
			}

			// Check if we're stepping on anyone's toes...
			if (file.exists()) {
				int answer = JOptionPane.showOptionDialog(this,file.getName()+" exists.  Do you want to overwrite the existing file?","Overwrite file?",0,JOptionPane.QUESTION_MESSAGE,null,new String [] {"Overwrite and Save","Cancel"},"Overwrite and Save");

				if (answer > 0) {
					return null;
				}
			}
			return file;
		}
	}

}
