/**
 * Copyright Copyright 2007-8 Simon Andrews
 *
 *    This file is part of StackMeasure.
 *
 *    StackMeasure is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    StackMeasure is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with StackMeasure; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
import ij.ImagePlus;
import ij.ImageStack;
import ij.gui.NewImage;

import java.util.Enumeration;
import java.util.Vector;

import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;


public class FileList {

	private Vector files = new Vector();
	private FileTableModel model = new FileTableModel();
	private ImagePlus ip = null;
	
	public FileList () {
		
	}
	
	public TableModel getTableModel () {
		return model;
	}
	
	public void addFile (StackMeasureFile file) {
		// We need to check that this file matches the dimensions
		// of any previously entered files.
		if (files.size() > 0) {
			StackMeasureFile ref = (StackMeasureFile)files.elementAt(0);
			
			if (file.getImagePlus().getHeight() != ref.getImagePlus().getHeight() || file.getImagePlus().getWidth() != ref.getImagePlus().getWidth() ||file.getImagePlus().getStackSize() != ref.getImagePlus().getStackSize()) {
				throw new IllegalArgumentException("File "+file+" doesn't match the dimensions of the previously entered file "+ref);
			}
		}
		
		if (file != null && !files.contains(file)) {
			if (ip != null) {
				ip.close();
			}
			ip = null;
			files.add(file);
			model.fireTableDataChanged();
		}
	}

	public void removeFile (StackMeasureFile file) {
		if (file != null && files.contains(file)) {
			if (ip != null) {
				ip.close();
			}
			ip = null;
			files.remove(file);
			file.getImagePlus().close();
			model.fireTableDataChanged();
		}
	}

	public int getHeight () {
		if (files.size()>0) {
			return ((StackMeasureFile)files.elementAt(0)).getImagePlus().getHeight();
		}
		else {
			return 0;
		}
	}
	
	public int getWidth () {
		if (files.size()>0) {
			return ((StackMeasureFile)files.elementAt(0)).getImagePlus().getWidth();
		}
		else {
			return 0;
		}
		
	}
	
	public int getSlices () {
		if (files.size()>0) {
			return ((StackMeasureFile)files.elementAt(0)).getImagePlus().getStackSize();
		}
		else {
			return 0;
		}
	}
	
	private class FileTableModel extends AbstractTableModel {

		public int getColumnCount() {
			return 2;
		}

		public int getRowCount() {
			return files.size();
		}

		public String getColumnName (int col) {
			if (col == 0) {
				return "File";
			}
			if (col == 1) {
				return "Colour";
			}
			return null;
		}
		
		public Object getValueAt(int row, int col) {
			if (col == 0) {
				return files.elementAt(row);
			}
			if (col == 1) {
				if (((StackMeasureFile)files.elementAt(row)).isRGB()) {
					return new String ("RGB");
				}
				else {
					return ((StackMeasureFile)files.elementAt(row)).getColour();
				}
			}
			return null;
		}
	}
	
	public ImagePlus getMergedImage () {
		if (ip != null) {
			return ip;
		}
		
		ip = NewImage.createRGBImage("Merged Image", getWidth(), getHeight(), getSlices(), NewImage.FILL_BLACK);
				
		RGB[][][] rgb = new RGB[getWidth()][getHeight()][getSlices()];
		// Initialise this new rbg stack
		for (int s=0;s<getSlices();s++) {
			for (int w=0;w<getWidth();w++) {
				for (int h=0;h<getHeight();h++) {
					rgb[w][h][s] = new RGB();
				}
			}
		}
		
		Enumeration e = files.elements();
		while (e.hasMoreElements()) {
			StackMeasureFile file = (StackMeasureFile)e.nextElement();
			if (file.isRGB()) {
				mergeRGB(rgb,file);
			}
			else {
				mergeBW(rgb,file);
			}
		}
		
		// Now convert the rgb values into an int to make the full pixel array

		for (int s=0;s<getSlices();s++) {
			int [] pixels = new int[getWidth()*getHeight()];
			for (int w=0;w<getWidth();w++) {
				for (int h=0;h<getHeight();h++) {
					pixels[w+(h*getWidth())] = rgb[w][h][s].getInt();
				}
			}
			if (s==0) {
				// For some reason we need to set the pixels for the
				// first slice via the image processor, rather than
				// the image stack.  If we don't do this then the first
				// slice ends up blank.
				ip.getProcessor().setPixels(pixels);	
			}
			else {
				ip.getImageStack().setPixels(pixels, s+1);
			}
		}
				
		return ip;
	}
	
	private void mergeRGB(RGB[][][] rgb, StackMeasureFile file) {
		ImageStack stack = file.getImagePlus().getImageStack();
		
		for (int s=0;s<getSlices();s++) {
			// Now we need to get the data for this slice
			int [] pixelData = (int [])stack.getPixels(s+1);

			for (int w=0;w<rgb.length;w++) {
				for (int h=0;h<rgb[w].length;h++) {
					// We need to split the integer value into its RGB components
					byte [] b = Pixel.intToByteArray(pixelData[w+(h*getWidth())]);

					RGB r = rgb[w][h][s];
					r.r+=b[0]&0xff;
					r.g+=b[1]&0xff;
					r.b+=b[2]&0xff;
				}
			}
		}
	}

	private void mergeBW(RGB[][][] rgb, StackMeasureFile file) {
		ImageStack stack = file.getImagePlus().getImageStack();

		int baseR = file.getColour().getRed();
		int baseG = file.getColour().getGreen();
		int baseB = file.getColour().getBlue();
		
		
		for (int s=0;s<stack.getSize();s++) {

			// Now we need to get the data for this slice
			Object o = stack.getPixels(s+1);
			
			if (o instanceof byte []) {
				byte [] a = (byte [])o;
				// It's an 8-bit greyscale image
				
				for (int w=0;w<rgb.length;w++) {
					for (int h=0;h<rgb[w].length;h++) {
						// We need to split the integer value into its RGB components
						byte b = a[w+(h*getWidth())];

						float correction = (b & 0xff) / 255f;
						
						RGB r = rgb[w][h][s];
						r.r+=(int)(baseR*correction);
						r.g+=(int)(baseG*correction);
						r.b+=(int)(baseB*correction);
					}
				}
				
			}
			else if (o instanceof short []) {
				short [] a = (short [])o;
				// It's a 16 bit greyscale image
				for (int w=0;w<rgb.length;w++) {
					for (int h=0;h<rgb[w].length;h++) {
						// We need to split the integer value into its RGB components
						short b = a[w+(h*getWidth())];

						float correction = b / 65536f;
						
						RGB r = rgb[w][h][s];
						r.r+=(int)(baseR*correction);
						r.g+=(int)(baseG*correction);
						r.b+=(int)(baseB*correction);
					}
				}
			}
			
			else {
				System.err.println("Unknown greyscale file type which produces "+o.getClass());
				return;
			}
			
		}
		
	}

	
	public int getFileCount () {
		return files.size();
	}
	
	public StackMeasureFile [] getFiles () {
		return (StackMeasureFile [])files.toArray(new StackMeasureFile[0]);
	}
	
	private class RGB {
		public int r = 0;
		public int g = 0;
		public int b = 0;
		
		public int getInt () {
			int i = 0;
			if (r>255) r=255;
			if (g>255) g=255;
			if (b>255) b=255;
			i=b;
			i+= g<<8;
			i+= r<<16;
//			System.out.println("I="+i);
			return i;
		}
	}
}
